#include <iostream>

using namespace std;

char **graph;
int nn, bd;

void init_graph(char val = '0') {
  graph = new char*[nn];
  for (int i = 0; i < nn; i++) {
    graph[i] = new char[nn];
    for (int j = i + 1; j < nn; j++)
      graph[i][j] = val;
  }
}

void blank(int _nn, int _bd) {
  nn = _nn;
  bd = _bd;
  init_graph();
}

void k(int _nn, int _bd) { // connected graph
  nn = _nn;
  bd = _bd;
  init_graph('-');
}

void star(int _bd, int) {
  bd = _bd;
  nn = bd + 1;
  init_graph();
  for (int i = 0; i < bd; i++)
    graph[i][bd] = '-';
}

void circles(int circles, int rays) { // circles and rays
  bd = rays * 2;
  nn = bd + rays * circles;
  init_graph();

  // outer boundary
  for (int i = 0; i < rays; i++)
    graph[i][nn - rays + i] = '-';

  // inner boundary
  for (int i = rays; i < 2*rays; i++)
    graph[i][rays + i] = '-';

  // circles
  for (int i = 0; i < circles; i++)
    for (int j = 0; j < rays; j++) {
      if (j == 0) // first node on each circle is connected to last on same
        graph[bd + i*rays][bd + (i+1)*rays - 1] = '-';
      if (j < rays - 1) // all but last node are connected to the one after them
        graph[bd + i*rays + j][bd + i*rays + j + 1] = '-';
      if (i < circles - 1) // all but last circle are connected to next circle
        graph[bd + i*rays + j][bd + (i+1)*rays + j] = '-';
    }
}

void lattice(int x, int y) {
  bd = 2*(x + y);
  nn = bd + x*y;
  init_graph();

  // top
  for (int i = 0; i < x; i++)
    graph[i][bd + i] = '-';

  // left side
  for (int i = 0; i < y; i++)
    graph[x + i][bd + x*i] = '-';

  // right side
  for (int i = 0; i < y; i++)
    graph[x + y + i][bd + x*(i+1) - 1] = '-';

  // bottom
  for (int i = 0; i < x; i++)
    graph[x + 2*y + i][nn - x + i] = '-';

  // interior
  for (int i = 0; i < y; i++)
    for (int j = 0; j < x; j++) {
      if (j < x - 1)
        graph[bd + x*i + j][bd + x*i + j + 1] = '-';
      if (i < y - 1)
        graph[bd + x*i + j][bd + x*(i+1) + j] = '-';
    }
}

void well(int _bd, int) { // well-connected graph
  bd = _bd;
  int m = (bd-1) / 4;

  nn = bd * (m+1);
  if (bd % 4 == 0) nn++;
  init_graph();
  if (bd % 4 == 0) nn--;

  for (int i = bd; i < nn; i++) {
    // radial semgent
    graph[i-bd][i] = '-';
    // circular segment
    if (i % bd != bd - 1)
      graph[i][i+1] = '-';
    else
      graph[i-bd+1][i] = '-';
  }

  switch (bd % 4) {
    case 0 :
      // boundary-to-boundary
      for (int i = 0; i < bd/2; i += 2)
        graph[i][i+1] = '-';
      for (int i = bd/2 + 1; i < bd-1; i += 2)
        graph[i][i+1] = '-';
      graph[0][bd-1] = '-';

      // hub in the middle
      for (int i = nn - bd; i < nn; i++)
        graph[i][nn] = '-';
      nn++;
      break;

    case 1 :
      // we're good
      break;

    case 2 :
      // boundary-to-boundary
      for (int i = 0; i < bd; i += 2)
        graph[i][i+1] = '-';
      break;

    case 3 :
      // boundary-to-boundary
      for (int i = 0; i < bd-1; i++)
        graph[i][i+1] = '-';
      graph[0][bd-1] = '-';
      break;
  }
}

void poly(int n, int) { // polygon in polygon
  bd = 2*n;
  nn = 3*n;
  init_graph();

  for (int i = 0; i < n; i++) {
    graph[i][i + 2*n] = '-';
    graph[i + n][i + 2*n] = '-';

    if (i == 0) {
      graph[i][nn - 1] = '-';
      graph[i + n][nn - 1] = '-';
    } else {
      graph[i][i + 2*n - 1] = '-';
      graph[i + n][i + 2*n - 1] = '-';
    }
  }
}

# define n_commands 7
struct command {
  char *cmd;
  int min1, min2;
  bool die_if_1_lt_2;
  char *err_msg;
  void (*proc)(int, int);
} commands[n_commands] = {
  { "blank",   2, 2, true,  "mkgraph blank (total_nodes) (boundary_nodes)", blank },
  { "k",       2, 2, true,  "mkgraph k (total_nodes) (boundary_nodes)",     k },
  { "star",    2, 0, false, "mkgraph star (boundary_nodes)",                star },
  { "circles", 1, 3, false, "mkgraph circles (circles) (rays)",             circles },
  { "lattice", 1, 1, false, "mkgraph lattice (width) (height)",             lattice },
  { "well",    2, 0, false, "mkgraph well (boundary_nodes)",                well },
  { "poly",    3, 0, false, "mkgraph poly (sides)",                         poly },
};

int main(int argc, char **argv) {
  // syntax checking is ugly, but necessary
  if (argc >= 3)
    for (int i = 0; i < n_commands; i++) {
      command *c = &commands[i];
      if (strcmp(argv[1], c->cmd) == 0) {
        if ((c->min2 == 0 && argc == 3) || (c->min2 != 0 && argc == 4)) {
          int arg1 = atoi(argv[2]), arg2 = c->min2 == 0 ? 0 : atoi(argv[3]);
          if (arg1 >= c->min1 && arg2 >= c->min2 && (!c->die_if_1_lt_2 || arg1 >= arg2)) {
            c->proc(arg1, arg2);
            goto success;
          }
        }
        cerr << "usage: " << c->err_msg << endl;
        return 1;
      }
    }

  cerr << "usage:" << endl;
  for (int i = 0; i < n_commands; i++)
    cerr << "  " << commands[i].err_msg << endl;
  return 1;

success:
  cout << nn << " " << bd << endl;
  for (int i = 0; i < nn; i++) {
    for (int j = 0; j < nn; j++)
      if (j <= i)
        cout << "  ";
      else
        cout << " " << graph[i][j];
    cout << endl;
  }

  for (int i = 0; i < nn; i++)
    delete[] graph[i];
  delete[] graph;

  return 0;
}

